@@ -85,6 +85,9 @@ class Reference(MiniLinq):
     def to_jvalue(self):
         return {'Ref': self.ref}
 
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__, self.ref)
+
 class Literal(MiniLinq):
     """
     An MiniLinq wrapper around a python value. Returns exactly the
@@ -101,6 +104,9 @@ class Literal(MiniLinq):
     def __eq__(self, other):
         return isinstance(other, Literal) and self.v == other.v
 
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__, self.v)
+
     @classmethod
     def from_jvalue(cls, jvalue):
         return cls(jvalue['Lit'])
@@ -129,18 +135,24 @@ class Bind(MiniLinq):
     def __eq__(self, other):
         return isinstance(other, Bind) and self.name == other.name and self.value == other.value and self.body == other.body
 
+    def __repr__(self):
+        return '%s(name=%r, value=%r, body=%r)' % (self.__class__.__name__, self.name, self.value, self.body)
+
     @classmethod
     def from_jvalue(cls, jvalue):
         fields = jvalue['Bind']
-        return cls(name=field['name'],
-                   value=MiniLinq.from_jvalue(field['value']),
-                   body=MiniLinq.from_jvalue(field['body']))
+        return cls(name=fields['name'],
+                   value=MiniLinq.from_jvalue(fields['value']),
+                   body=MiniLinq.from_jvalue(fields['body']))
 
     def to_jvalue(self):
         return {'Bind':{'name': self.name,
                         'value': self.value.to_jvalue(),
                         'body': self.body.to_jvalue()}}
 
+    def __repr__(self):
+        return '%s(name=%r, value=%r, body=%r)' % (self.__class__.__name__, self.name, self.value, self.body)
+
 
 class Filter(MiniLinq):
     """
@@ -178,10 +190,14 @@ class Filter(MiniLinq):
                    name   = fields.get('name'))
 
     def to_jvalue(self):
-        return {'Filter': {'predicate': self.predicate,
-                           'source': self.source,
+        return {'Filter': {'predicate': self.predicate.to_jvalue(),
+                           'source': self.source.to_jvalue(),
                            'name': self.name}}
 
+    def __repr__(self):
+        return '%s(source=%r, name=%r, predicate=%r)' % (self.__class__.__name__, self.source, self.name, self.predicate)
+
+
 class List(MiniLinq):
     """
     A list of expressions, embeds the [ ... ] syntax into the
@@ -227,7 +243,7 @@ class Map(MiniLinq):
         def iterate(env=env, source_result=source_result): # Python closure workaround
             if self.name:
                 for item in source_result:
-                    yield self.body.eval(env.bind(name, item))
+                    yield self.body.eval(env.bind(self.name, item))
             else:
                 for item in source_result:
                     yield self.body.eval(env.replace(item)) 
@@ -299,8 +315,8 @@ class FlatMap(MiniLinq):
                    name   = fields.get('name'))
 
     def to_jvalue(self):
-        return {'FlatMap': {'body': self.body,
-                            'source': self.source,
+        return {'FlatMap': {'body': self.body.to_jvalue(),
+                            'source': self.source.to_jvalue(),
                             'name': self.name}}
 
 class Apply(MiniLinq):
@@ -333,6 +349,9 @@ class Apply(MiniLinq):
         return {'Apply': {'fn': self.fn.to_jvalue(),
                           'args': [arg.to_jvalue() for arg in self.args]}}
 
+    def __repr__(self):
+        return '%s(%r, *%r)' % (self.__class__.__name__, self.fn, self.args)
+
 
 class Emit(MiniLinq):
     """
@@ -403,3 +422,4 @@ MiniLinq.register(FlatMap)
 MiniLinq.register(Apply)
 MiniLinq.register(Emit)
 MiniLinq.register(List)
+MiniLinq.register(Bind)
